package com.cweb.controller;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.AESEncodeUtil;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.*;
import com.hai.config.RechargeConfig;
import com.hai.entity.*;
import com.hai.enum_type.DiscountUseScope;
import com.hai.model.*;
import com.hai.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

@Controller
@RequestMapping(value = "/outRechargeOrder")
@Api(value = "充值订单接口")
public class OutRechargeOrderController {
    private static Logger log = LoggerFactory.getLogger(HighMerchantStoreController.class);

    @Autowired
    private UserCenter userCenter;

    @Resource
    private OutRechargeOrderService outRechargeOrderService;

    @Resource
    private OutRechargePriceService outRechargePriceService;

    @Resource
    private SecConfigService secConfigService;

    @Resource
    private HighUserPayPasswordService highUserPayPasswordService;

    @Resource
    private HighDiscountUserRelService highDiscountUserRelService;

    @Resource
    private BsConfigService bsConfigService;

    @Resource
    private RechargeConfig rechargeConfig;

    @RequestMapping(value="/addOrder",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "增加订单")
    public ResponseData addOrder(@RequestBody JSONObject object, HttpServletRequest request) {
        try {

            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();


            if (StringUtils.isBlank(object.getString("rechargeContent")) ||
                StringUtils.isBlank(object.getString("regionId")) ||
                    object.getLong("goodsId") == null
            ) {
                log.error("addOrder error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            OutRechargeOrder outRechargeOrder = new OutRechargeOrder();

            // 产品id
            Long goodsId = object.getLong("goodsId");

            // 充值内容
            String rechargeContent = object.getString("rechargeContent");

            // 查询产品详情
            OutRechargePriceModel outRechargePrice = outRechargePriceService.findById(goodsId , null);

            Map<String, Object> listMap = new HashMap<>();
            listMap.put("productType", "3");
            listMap.put("returnType", 1);
            listMap.put("sourceId", goodsId);

            // 查询产品积分抵扣比例
            BsProductDiscount bsProductDiscount = bsConfigService.getProductDiscountByMap(listMap);

            // 判断充值系统是否关闭
            if (!secConfigService.isConfig("RECHARGE" , "1")) {
                log.error("addOrder error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.RECHARGE_CLOSE, "");
            }

            HighDiscountUserRel highDiscountUserRel = null;
            // 判断是否有优惠券
            if (object.getLong("memDiscountId") != null) {
                // 卡优惠券信息
                highDiscountUserRel = highDiscountUserRelService.getRelById(object.getLong("memDiscountId"));
                if (highDiscountUserRel == null || highDiscountUserRel.getStatus() != 1) {
                    log.error("HighOrderController --> addOrder() error!", "优惠券状态错误");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "优惠券状态错误");
                }
                if (!highDiscountUserRel.getHighDiscount().getUseScope().equals(DiscountUseScope.type1.getType())
                     && !highDiscountUserRel.getHighDiscount().getUseScope().equals(DiscountUseScope.type3.getType())) {
                    log.error("HighOrderController --> addOrder() error!", "无法使用此优惠券");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法使用此优惠券");
                }
            }

            // 优惠券抵扣
            if (highDiscountUserRel != null) {
                outRechargeOrder.setMemDiscountName(highDiscountUserRel.getHighDiscount().getDiscountName());

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (highDiscountUserRel.getHighDiscount().getDiscountType() == 1) {
                    // 如果商品支付总额 小于 满减价格
                    if (outRechargePrice.getPayPrice().compareTo(highDiscountUserRel.getHighDiscount().getDiscountCondition()) > 1) {
                        log.error("HighOrderController --> addOrder() error!", "订单未达到满减额度");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "订单未达到"+highDiscountUserRel.getHighDiscount().getDiscountCondition()+"元,无法使用此优惠券");
                    }
                    // 计算支付金额 = 商品支付总额 - 满减额度
                    BigDecimal payPrice = outRechargePrice.getPayPrice().subtract(highDiscountUserRel.getHighDiscount().getDiscountPrice());

                    outRechargeOrder.setDiscountDeductionPrice(highDiscountUserRel.getHighDiscount().getDiscountPrice());
                    // 如果总额小于0
                    if (payPrice.compareTo(new BigDecimal("0")) < 0) {
                        outRechargeOrder.setPayRealPrice(new BigDecimal("0"));
                    } else {
                        outRechargeOrder.setPayRealPrice(payPrice);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (highDiscountUserRel.getHighDiscount().getDiscountType() == 2) {
                    // 计算支付金额 = 商品支付总额 - 满减额度
                    BigDecimal payPrice = outRechargePrice.getPayPrice().subtract(highDiscountUserRel.getHighDiscount().getDiscountPrice());
                    outRechargeOrder.setDiscountDeductionPrice(highDiscountUserRel.getHighDiscount().getDiscountPrice());

                    // 如果总额小于0
                    if (payPrice.compareTo(new BigDecimal("0")) < 0) {
                        outRechargeOrder.setPayRealPrice(new BigDecimal("0"));
                    } else {
                        outRechargeOrder.setPayRealPrice(payPrice);
                    }
                }

                // 卡卷类型  1：满减  2：抵扣   3：折扣
                if (highDiscountUserRel.getHighDiscount().getDiscountType() == 3) {
                    BigDecimal discountPrice = highDiscountUserRel.getHighDiscount().getDiscountPrice();
                    // 订单总额 * 折扣
                    BigDecimal payPrice = outRechargePrice.getPayPrice().multiply(discountPrice);
                    outRechargeOrder.setDiscountDeductionPrice(outRechargePrice.getPayPrice().subtract(payPrice));
                    outRechargeOrder.setPayRealPrice(payPrice);
                }
            }

            // 判断积分数量是否大于0
            if (object.getLong("integralNum") > 0 && bsProductDiscount.getDiscount().compareTo(new BigDecimal("0")) > 0) {


                // 判断用户积分是否够
                if (object.getLong("integralNum") > userInfoModel.getHighUser().getGold()) {
                    log.error("HighOrderController --> addOrder() error!", "积分大于用户积分额度");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "用户积分" + userInfoModel.getHighUser().getGold());
                }

                // 积分抵扣金额
                BigDecimal integralDeductionPrice = BigDecimal.valueOf(object.getLong("integralNum") / 100);
                // 最高可抵扣金额
                BigDecimal maxIntegralDeductionPrice = outRechargeOrder.getPayRealPrice().multiply(bsProductDiscount.getDiscount()).divide(new BigDecimal(100)).setScale(2, RoundingMode.HALF_DOWN);
                // 判读积分是否大于限制额度
                if (maxIntegralDeductionPrice.compareTo(integralDeductionPrice) > 0) {
                    log.error("HighOrderController --> addOrder() error!", "积分大于限制额度");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "订单最大抵扣积分数量" + object.getLong("integralNum"));
                }

                // 判断积分抵扣比例是否为100% 并且积分数量是否可以抵扣最后的支付金额
                if (bsProductDiscount.getDiscount().compareTo(new BigDecimal(100)) == 0 && integralDeductionPrice.compareTo(outRechargeOrder.getPayRealPrice()) == 0) {
                    // 查询用户支付密码
                    HighUserPayPassword userPayPassword = highUserPayPasswordService.getDetailByUser(userInfoModel.getHighUser().getId());
                    if (userPayPassword == null) {
                        log.error("orderToPay error!", "未设置支付密码");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_SET_USER_PAY_PWD, "");
                    }
                    if (StringUtils.isBlank(object.getString("password"))) {
                        log.error("orderToPay error!", "未输入支付密码");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_ENTER_USER_PAY_PWD, "");
                    }
                    // 校验支付密码
                    if (!AESEncodeUtil.aesEncrypt(object.getString("password")).equals(userPayPassword.getPassword())) {
                        log.error("orderToPay error!", "");
                        throw ErrorHelp.genException(SysCode.System, ErrorCode.USER_PAY_PWD_ERROR, "");
                    }
                }

                outRechargeOrder.setIntegralDeductionPrice(integralDeductionPrice);
                outRechargeOrder.setPayRealPrice(outRechargeOrder.getPayRealPrice().subtract(outRechargeOrder.getIntegralDeductionPrice()));

            }

            outRechargeOrder.setUserId(userInfoModel.getHighUser().getId());
            outRechargeOrder.setRechargeContent(rechargeContent);
            outRechargeOrder.setRegionId(object.getString("regionId"));
            outRechargeOrder.setUserName(userInfoModel.getHighUser().getName());
            outRechargeOrder.setRechargeType(outRechargePrice.getRechargeType());
            outRechargeOrder.setUserPhone(userInfoModel.getHighUser().getPhone());
            outRechargeOrder.setOrderNo("RCG" + DateUtil.date2String(new Date(),"yyyyMMddHHmmss") + IDGenerator.nextId(5));
            outRechargeOrder.setCreateTimed(new Date());

            // 判断积分支付是否扣完金额
            if (outRechargeOrder.getPayRealPrice().compareTo(new BigDecimal(0)) == 0) {
                // 201:充值中 202：充值成功 203：充值失败 204：未充值
                outRechargeOrder.setRechargeStatus(201);
                // 101.待支付 102.已支付 100.已完成 104.已取消 105.已退款
                outRechargeOrder.setPayStatus(102);
            } else {
                // 201:充值中 202：充值成功 203：充值失败 204：未充值
                outRechargeOrder.setRechargeStatus(204);
                // 101.待支付 102.已支付 100.已完成 104.已取消 105.已退款
                outRechargeOrder.setPayStatus(101);
            }

            outRechargeOrderService.insertOrder(outRechargeOrder);
            return ResponseMsgUtil.success(outRechargeOrder);

        } catch (Exception e) {
            log.error("HighOrderController -> addOrder() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/getOrderById", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id查询订单详情")
    public ResponseData getOrderById(@RequestParam(name = "orderId", required = true) Long orderId) {
        try {

            return ResponseMsgUtil.success(outRechargeOrderService.findByOrderId(orderId));

        } catch (Exception e) {
            log.error("HighOrderController --> getOrderById() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/cancelOrder", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "取消订单")
    public ResponseData cancelOrder(@RequestParam(name = "orderId", required = true) Long orderId) {
        try {

            OutRechargeOrder outRechargeOrder  = outRechargeOrderService.findByOrderId(orderId);

            if (outRechargeOrder.getRechargeStatus() == 1) {
                outRechargeOrderService.cancelOrder(orderId);
            } else {
                log.error("orderToPay error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.STATUS_ERROR, "");
            }

            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error("HighOrderController --> cancelOrder() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getUserOrderList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "获取用户订单")
    public ResponseData getUserOrderList(
            @RequestParam(name = "status", required = false) Integer status,
            @RequestParam(name = "orderNo", required = false) String orderNo,
            @RequestParam(name = "rechargeModel", required = false) String rechargeModel,
            @RequestParam(name = "pageNum", required = true) Integer pageNum,
            @RequestParam(name = "pageSize", required = true) Integer pageSize,
            HttpServletRequest request) {
        try {

            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();

            Map<String,String> map = new HashMap<>();
            map.put("userId", userInfoModel.getHighUser().getId().toString());
            if (status != null) {
                map.put("status", status.toString());
            }
            if (StringUtils.isNotBlank(orderNo)) {
                map.put("orderNo", orderNo);
            }
            if (StringUtils.isNotBlank(rechargeModel)) {
                map.put("rechargeModel", rechargeModel);
            }

            PageHelper.startPage(pageNum,pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(outRechargeOrderService.getListRechargeOrder(map)));

        } catch (Exception e) {
            log.error("HighOrderController --> getUserOrderList() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


}
